'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _join = require('path');

var _readFileSync = require('fs');

var _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not = require('ramda');

var _Debug = require('debug');

var _Debug2 = _interopRequireWildcard(_Debug);

var _Jade = require('jade');

var _Jade2 = _interopRequireWildcard(_Jade);

exports['default'] = create;

var read = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.partialRight(_readFileSync.readFileSync, 'utf8');
var path = _join.join.bind(null, __dirname);
path.template = path.bind('../templates');
var log = _Debug2['default']('webpack-plugin-bootstrap');

var defaultConfig = {
  output: {
    fileName: 'index.html'
  },
  locals: {},
  compile: {
    pretty: true,
    doctype: 'html'
  }
};

function create() {
  var customConfig = arguments[0] === undefined ? {} : arguments[0];

  var config = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.merge(defaultConfig, customConfig);
  log('config', config);

  return { apply: apply };

  function apply(compiler) {
    log('Hello World!');
    compiler.plugin('emit', function (c, cb) {
      var stats = c.getStats().toJson();
      var publicPath = getPublicPath(c);
      var prependPublicPath = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.concat(publicPath);
      log('compilation', Object.keys(c));
      log('options', Object.keys(c.options));
      log('_plugins', c._plugins);
      log('stats', Object.keys(stats));
      log('options.output', c.options.output);
      log('options.devServer', c.options.devServer);
      log('stats.assets', stats.assets);
      log('publicPath', publicPath);
      log('hotUpdateChunkTemplate', c.hotUpdateChunkTemplate);

      var jsAssets = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.flatten(_merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.map(getPublicAssets, _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.values(stats.assetsByChunkName)));
      log('jsAssets', jsAssets);

      /* If using hot-enabled webpack-dev-server then add its necessary runtime script
      to jsAssets so that it is automatically included in our final html. */

      if (c.options.devServer && c.options.devServer.hot) {
        jsAssets.push('/webpack-dev-server.js');
      }

      /* Resolve template locals. */

      var locals = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.merge({
        assetTypesAssetPaths: {
          js: jsAssets
        },
        title: 'App',
        headers: {
          mobile: true,
          utf8: true
        },
        react: {
          mainGlobalIdentifier: 'Main'
          // + Dynamically Resolved Fields.
        },
        mainElement: {
          id: 'main'
        }
      }, config.locals);

      locals.mainQuerySelector = locals.mainElement ? '#' + locals.mainElement.id : 'body';

      /* Generate HTML contents. */

      var templateSource = read(path.template('default.jade'));
      log('template', templateSource);

      var template = _Jade2['default'].compile(templateSource, config.compile);
      var html = template(locals);
      log('html', html);

      /* Add HTML asset to build. */

      c.assets[config.output.fileName] = {
        source: function source() {
          return html;
        },
        size: function size() {
          return html.length;
        }
      };

      cb();

      function getPublicAssets(assets) {
        return _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.map(prependPublicPath, _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.filter(isntSourceMapFile, assets));
      }
    });
  }
}

var isntSourceMapFile = _merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.compose(_merge$partialRight$flatten$concat$values$mapObj$map$compose$filter$not.not, isSourceMapFile);

function isSourceMapFile(fileName) {
  return /\.map$/.test(fileName);
}

function getPublicPath(compilation) {
  var options = compilation.options;
  var devServer = options.devServer;

  if (devServer.publicPath) {
    return devServer.publicPath;
  }var publicPath = options.publicPath;

  if (publicPath) {
    return publicPath;
  }return '/';
}
module.exports = exports['default'];